/* hfload.c
 *
 * This file is subject to the terms and conditions of the GNU
 * General Public License.  See the file "COPYING" in the main
 * directory of this archive for more details.
 *
 * Copyright (C) 2000, Jay Carlson
 */

/*
 * Boot loader main program.
 */
#include <linux/config.h>

#include <unistd.h>
#include <linux/elf.h>
#include "hfload.h"
#include <asm/rtl865x/rtl865xc_asicregs.h>

#if defined(CONFIG_RTL865X) || defined(CONFIG_RTL8196B)
	#define BASE_ADDR 0xB8000000
	#define set_io_port_base(base)	\
		do { * (unsigned long *) &mips_io_port_base = (base); } while (0)	
	
	const unsigned long mips_io_port_base;
#endif


int file_offset;

int old_stack_pointer;

#define MAX_PHDRS_SIZE 8

Elf32_Ehdr header;
Elf32_Phdr phdrs[MAX_PHDRS_SIZE];

extern void flush_cache(void);

void
zero_region(char *start, char *end)
{
	char *addr;
	int count;

	count = end - start;
#ifndef __DO_QUIET__
	printf("zeroing from %08x to to %08x, 0x%x bytes\n", start, end, count);
#endif

#ifndef FAKE_COPYING
	memset(start, 0, count);
#endif
}

void
load_phdr(Elf32_Phdr *phdr)
{
	char *addr, *end;
	
	seek_forward(phdr->p_offset);
	
	addr = (char *)phdr->p_vaddr;
	end = ((char *)addr) + phdr->p_memsz;
	
	copy_to_region(addr, phdr->p_filesz);
	
	addr = ((char *)addr) + phdr->p_filesz;
	
	zero_region(addr, end);
}

void main(unsigned long stack_start_addr)
{
	int i;
	file_offset = 0;
	{
		RTL_W32(PCRP0, ((RTL_R32(PCRP0))&(~EnablePHYIf)) ); /* Disable PHY IF */
		RTL_W32(PCRP1, ((RTL_R32(PCRP1))&(~EnablePHYIf)) ); /* Disable PHY IF */
		RTL_W32(PCRP2, ((RTL_R32(PCRP2))&(~EnablePHYIf)) ); /* Disable PHY IF */
		RTL_W32(PCRP3, ((RTL_R32(PCRP3))&(~EnablePHYIf)) ); /* Disable PHY IF */
		RTL_W32(PCRP4, ((RTL_R32(PCRP4))&(~EnablePHYIf)) ); /* Disable PHY IF */
		RTL_W32(PCRP4, ((RTL_R32(PCRP5))&(~EnablePHYIf)) ); /* Disable PHY IF */
		RTL_W32(PCRP4, ((RTL_R32(PCRP6))&(~EnablePHYIf)) ); /* Disable PHY IF */
		RTL_W32(PCRP4, ((RTL_R32(PCRP7))&(~EnablePHYIf)) ); /* Disable PHY IF */
		RTL_W32(PCRP4, ((RTL_R32(PCRP8))&(~EnablePHYIf)) ); /* Disable PHY IF */
	}
#ifdef CONFIG_RTL8186_KB
//#if defined(CONFIG_RTL8186_KB_N)
//	RTL_W32(0xb8000030, (RTL_R32(0xb8000030) | (0xfffff80)));
//#endif
#if defined(CONFIG_RTL8186_KB_N)
        //printk("%s(%d)\n",__FUNCTION__,__LINE__);//Added for test
        //unsigned int mux_mask;
        /*      A4      */
        RTL_W32(PABCD_CNR,(RTL_R32(PABCD_CNR)&(~(0x10 << 0))));
        RTL_W32(PABCD_DIR, (RTL_R32(PABCD_DIR) | (0x10 << 0)));
        /*      B0~B3   */
        RTL_W32(PABCD_CNR,(RTL_R32(PABCD_CNR)&(~(0x0f << 8))));
        RTL_W32(PABCD_DIR, (RTL_R32(PABCD_DIR) | (0x0f << 8)));
        /*      C0~C4   */
        RTL_W32(PABCD_CNR,(RTL_R32(PABCD_CNR)&(~(0x1f << 16))));
        RTL_W32(PABCD_DIR, (RTL_R32(PABCD_DIR) | (0x1f << 16)));

        /* A4(actually A0~A4), B0~B3, C0~C4 change to GPIO mode */
        //mux_mask=REG_IOCFG_JTAG | REG_IOCFG_LED_S0 | REG_IOCFG_LED_S1 |
        //              REG_IOCFG_LED_S2 | REG_IOCFG_LED_S3 | REG_IOCFG_LED_P0 |
        //              REG_IOCFG_LED_P1 | REG_IOCFG_LED_P2 | REG_IOCFG_LED_P3 |
        //              REG_IOCFG_LED_P4;
        //printk("%s(%d),PIN_MUX_SEL=0x%x,mux_mask=0x%x\n",__FUNCTION__,__LINE__,
        //      PIN_MUX_SEL,mux_mask);//Added for test
        //RTL_W32(PIN_MUX_SEL, (RTL_R32(PIN_MUX_SEL) | mux_mask));
        //printk("%s(%d),RTL_GPIO_MUX=0x%x,RTL_GPIO_MUX_DATA=0x%x\n",
        //      __FUNCTION__,__LINE__,RTL_GPIO_MUX,RTL_GPIO_MUX_DATA);//Added for test
#ifdef CONFIG_RTL8196B
        RTL_W32(PIN_MUX_SEL, (RTL_R32(PIN_MUX_SEL) | 0xfffff80));/*For Belkin_n board, not for demo board*/
#endif

        //all light on
        //RTL_W32(PABCD_DAT,(RTL_R32(PABCD_DAT)&(~(0x10 << 0))));//A4
        //RTL_W32(PABCD_DAT,(RTL_R32(PABCD_DAT)&(~(0x0f << 8))));//B0~B3
        //RTL_W32(PABCD_DAT,(RTL_R32(PABCD_DAT)&(~(0x1f << 16))));//C0~C4

        //all light off
        RTL_W32(PABCD_DAT,(RTL_R32(PABCD_DAT)|(0x10 << 0)));//A4
        RTL_W32(PABCD_DAT,(RTL_R32(PABCD_DAT)|(0x0f << 8)));//B0~B3
        RTL_W32(PABCD_DAT,(RTL_R32(PABCD_DAT)|(0x1f << 16)));//C0~C4
#else
	/* default brightness 100% */
	RTL_W32(PEFGH_DAT,((RTL_R32(PEFGH_DAT)&(~(0xF8<< 24)))|((0x80)<<24)));
	/* power blink blue */
	RTL_W32(PABCD_DAT,(RTL_R32(PABCD_DAT)|((0x1<< 24))));
	RTL_W32(LEDCR1, (RTL_R32(LEDCR1) | (0x2)));
	RTL_W32(LEDBCR, (RTL_R32(LEDBCR)|((0x2))));
#endif
#endif
#if defined(CONFIG_RTL865X) || defined(CONFIG_RTL8196B)
 	set_io_port_base(BASE_ADDR);
#endif

#ifndef __DO_QUIET__
	printf("decompressing kernel:\n");
#endif

#ifndef BZ2_COMPRESS
	decompress_kernel(UNCOMPRESS_OUT, stack_start_addr+4096, FREEMEM_END, 0);
#else
	decompress_kernel(UNCOMPRESS_OUT, stack_start_addr+4096, FREEMEM_END, 0);
#endif

#ifndef __DO_QUIET__
	printf("done decompressing kernel.\n");
#endif

	flush_cache();
	start_kernel(0x80000000);
}
