/* vi: set sw=4 ts=4: */

/*
 * @file Member ID Definition of Group LOCAL.
 *
 * Moderator: GeorgeWang/RupertLi
 * Group ID: 0x00010000/0x80010000
 */

#ifndef _CSID_LOCAL_H
#define _CSID_LOCAL_H

#include "csid_gid.h"

#define _CFG_LOCAL(x)       (CSID_GIDC_LOCAL|(x))
#define _STA_LOCAL(x)       (CSID_GIDS_LOCAL|(x))

#define LOCAL_WANTYPE_DHCP              0x00
#define LOCAL_WANTYPE_NULL              0x01
#define LOCAL_WANTYPE_3G				0x10
#define LOCAL_WANTYPE_IBURST			0x11
#define LOCAL_WANTYPE_WIBRO             0x12
#define LOCAL_WANTYPE_WISP              0x13
#define LOCAL_WANTYPE_WLANCLI           0x14
#define LOCAL_WANTYPE_FLASHOFDM			0x15
#define LOCAL_WANTYPE_S3G             	0x16
#define LOCAL_WANTYPE_FIXED             0x20
#define LOCAL_WANTYPE_PPPOE             0x40
#define LOCAL_WANTYPE_PPPOE_MULTI       0x41
#define LOCAL_WANTYPE_PPPOA             0x50
#define LOCAL_WANTYPE_IPOA_DHCP         0x51
#define LOCAL_WANTYPE_IPOA_FIXED        0x52
#define LOCAL_WANTYPE_PPTP              0x60
#define LOCAL_WANTYPE_L2TP              0x80
#define LOCAL_WANTYPE_DIALUP			0x90
#define LOCAL_WANTYPE_DHCPD 			0x70
#define LOCAL_WANTYPE_AUTO	 			0x110

#define LOCAL_WAN_STATUS_DISCONNECT     0x00
#define LOCAL_WAN_STATUS_CONNECTING     0x01
#define LOCAL_WAN_STATUS_CONNECT        0x02
#define LOCAL_WAN_STATUS_DISCONNECTING  0x03
#define LOCAL_WAN_STATUS_WAIT_TRAFFIC	0x05
#define LOCAL_WAN_STATUS_AUTH_FAIL		0x06

/* Define maximum httpd auth username password length*/
#define LOCAL_HTTP_USER_PASSWD_LENGTH 32

/* Configuration Items */
#define CSID_C_LOCAL_HOSTNAME           _CFG_LOCAL(0x0001) //T=str, local host name
#define CSID_C_LOCAL_PASSWORD           _CFG_LOCAL(0x0002) //T=str, D="admin", administrator password
#define CSID_C_LOCAL_WANTYPE            _CFG_LOCAL(0x0003) //T=u32, D=0, wan type (default is DHCP)
#define CSID_C_LOCAL_LANIP              _CFG_LOCAL(0x0004) //T=ipv4, D={192.168.123.254}, lan ip
#define CSID_C_LOCAL_LANNM              _CFG_LOCAL(0x0005) //T=ipv4, D={255.255.255.0}, lan subnet mask
#define CSID_C_LOCAL_ROUTER_TYPE		_CFG_LOCAL(0x0006) //T=u32, D=0, router type, 0:normal(1WAN+4LAN) 1:single ethernet
#define CSID_C_LOCAL_SIG_ETHER_PORT     _CFG_LOCAL(0x0007) //T=u32, D=2, For the "only one" ethernet port exists 0:no function 1:WAN 2:LAN
#define CSID_C_LOCAL_HOMECOM_ADMIN_USERNAME		_CFG_LOCAL(0x0008) //T=str, D="admin", UI username
#define CSID_C_LOCAL_HOMECOM_ADMIN_PASSWORD		_CFG_LOCAL(0x0009) //T=str, D="admin", UI password
#define CSID_C_LOCAL_ETHER_AUTO_DETECT	_CFG_LOCAL(0x000A) //T=u32, D=0, 0:stop auto-detection of DHCP, 1: start it.
#define CSID_C_LOCAL_USERNAME			_CFG_LOCAL(0x000B) //T=u32, D="admin", Login username	
#define CSID_C_LOCAL_WAN_WEIGHT			_CFG_LOCAL(0x000C) //T=u8, wan weight
#define CSID_C_LOCAL_LOADBALANCE_POLICY	_CFG_LOCAL(0x000D) //T=u32, D=100, load balance policy
#define CSID_C_LOCAL_LOADBALANCE_MODE	_CFG_LOCAL(0x000E) //T=u32, D=0, load balance mode
#define CSID_C_LOCAL_WAN_MAC            _CFG_LOCAL(0x000F) //T=mac, MAC of WAN port

#define CSID_C_LOCAL_WLANENABLE         _CFG_LOCAL(0x0010) //T=u8, D=1, WLAN enable/disable (default is enable)
#define CSID_C_LOCAL_3G_ENABLE			_CFG_LOCAL(0x0011) //T=u32, D=0, 3G wantype enable/disable for UI page showing (default is disable)
#define CSID_C_LOCAL_LANWAN_CHANGE      _CFG_LOCAL(0x0012) //T=u8, D=0, 0:LAN Port 1:WAN Port
#define CSID_C_LOCAL_GUEST_PASSWORD     _CFG_LOCAL(0x0013) //T=str, D="guest", guest password
#define CSID_C_LOCAL_PASSWORD_TYPE      _CFG_LOCAL(0x0014) //T=u8, D=0, password type 0:admin 1:guest
#define CSID_C_LOCAL_INFO_ROM_ID		_CFG_LOCAL(0x0015) //T=str, romid check use
#define CSID_C_LOCAL_PREV_ETHER_WANTYPE		_CFG_LOCAL(0x0016) //T=u32, D=0, previous wan type (default is DHCP)
#define CSID_C_LOCAL_PREV_WIRELESS_WANTYPE	_CFG_LOCAL(0x0017) //T=u32, D=16, previous wan type (default is 3G)
#define CSID_C_LOCAL_WLAN2ENABLE         _CFG_LOCAL(0x0018) //T=u8, D=1, WLAN2 enable/disable (default is enable)
#define CSID_C_LOCAL_GUEST_USERNAME      _CFG_LOCAL(0x0019) //T=str, D="guest", guest username
#define CSID_C_LOCAL_SAVE_USERNAME       _CFG_LOCAL(0x001a) //T=str, save login username for mutil login

/* Time control */
#define CSID_C_LOCAL_RESET_TIME         _CFG_LOCAL(0x0020) //T=u32, D=6, press RESET button time
#define CSID_C_LOCAL_RESET_TIME_MAX     _CFG_LOCAL(0x0029) //T=u32, D=6, press RESET button time
#define CSID_C_LOCAL_WPS_LOCK_TIME      _CFG_LOCAL(0x0021) //T=u32, D=120, WPS enable time
#define CSID_C_LOCAL_WIFI_ONOFF_TIME    _CFG_LOCAL(0x0022) //T=u32, D=3, press WiFi on/off button time

/* WAN interface selection*/ /*refer to CDD531 */
#define CSID_C_LOCAL_WAN_INTERFACE		_CFG_LOCAL(0x0023) //T=u32, D=0, 0:ADSL 1:Ethernet 2:3G

/* USB type selection*/
#define CSID_C_LOCAL_USBTYPE            _CFG_LOCAL(0x0024) //T=u32, D=0, 0:Storage 1:Printer 2:3G

/* WPS button time control */
#define CSID_C_LOCAL_WPS_TIME			_CFG_LOCAL(0x0025) //T=u32, D=3, press WPS button time (MIN time)
#define CSID_C_LOCAL_WPS_TIME_MAX	    _CFG_LOCAL(0x002a) //T=u32, D=6, press WPS button time (MAC time)

#define CSID_C_LOCAL_WANTYPE_RES        _CFG_LOCAL(0x0026) //T=u32, D=0, wan type (default is DHCP)

/* DIP Switch mode */
#define CSID_C_LOCAL_WANTYPE_DSL_MODE   _CFG_LOCAL(0x0027) //T=u32, D=0, wan type (default is DHCP)
#define CSID_C_LOCAL_WANTYPE_3G_MODE    _CFG_LOCAL(0x0028) //T=u32, D=0, wan type (default is 3G)

/*Device is in AP Mode, refer to CDM532RP */
#define CSID_C_LOCAL_AP_IP				_CFG_LOCAL(0x0030) //T=ipv4, D={192.168.123.254}
#define CSID_C_LOCAL_AP_NM				_CFG_LOCAL(0x0031) //T=ipv4, D={255.255.255.0}
#define CSID_C_LOCAL_AP_GW				_CFG_LOCAL(0x0032) //T=ipv4, AP's GW
#define CSID_C_LOCAL_AP_PRIDNS			_CFG_LOCAL(0x0033) //T=ipv4, AP's primary dns
#define CSID_C_LOCAL_AP_SECDNS			_CFG_LOCAL(0x0034) //T=ipv4, AP's secondary dns
#define CSID_C_LOCAL_AP_CNT_TYPE		_CFG_LOCAL(0x0035) //T=u32, D=1,  0:Dynamic (DHCP Client) , 1:Fixed IP
#define CSID_C_LOCAL_AP_HOSTNAME		_CFG_LOCAL(0x0036) //T=str, AP's Hostname
#define CSID_C_LOCAL_AP_DEVICENAME		_CFG_LOCAL(0x0037) //T=str, AP's Device Name

/*Device is in STA Mode, refer to CDM532RP */
#define CSID_C_LOCAL_STA_IP				_CFG_LOCAL(0x0040) //T=ipv4, D={192.168.123.254}
#define CSID_C_LOCAL_STA_NM				_CFG_LOCAL(0x0041) //T=ipv4, D={255.255.255.0}
#define CSID_C_LOCAL_STA_GW				_CFG_LOCAL(0x0042) //T=ipv4, STA's GW
#define CSID_C_LOCAL_STA_PRIDNS			_CFG_LOCAL(0x0043) //T=ipv4, STA's primary dns
#define CSID_C_LOCAL_STA_SECDNS			_CFG_LOCAL(0x0044) //T=ipv4, STA's secondary dns
#define CSID_C_LOCAL_STA_CNT_TYPE		_CFG_LOCAL(0x0045) //T=u32, D=1,  0:Dynamic (DHCP Client) , 1:Fixed IP
#define CSID_C_LOCAL_STA_HOSTNAME		_CFG_LOCAL(0x0046) //T=str, STA's Hostname
#define CSID_C_LOCAL_STA_DEVICENAME		_CFG_LOCAL(0x0047) //T=str, STA's Device Name

/* for HOMECOM normal user */
#define CSID_C_LOCAL_HOMECOM_NORMAL_USERNAME	_CFG_LOCAL(0x0050) //T=str, UI username
#define CSID_C_LOCAL_HOMECOM_NORMAL_PASSWORD	_CFG_LOCAL(0x0060) //T=str, UI password
#define CSID_C_LOCAL_HOMECOM_NORMAL_AUTHORITY	_CFG_LOCAL(0x0070) //T=str, D="0", User authority

/* for CLI login configuration */
#define CSID_C_LOCAL_CLI_ACCOUNT           _CFG_LOCAL(0x0075) //T=str, D="", CLI username
#define CSID_C_LOCAL_CLI_PASSWORD           _CFG_LOCAL(0x0076) //T=str, D="", CLI password
#define CSID_C_LOCAL_ADMIN_ALIAS           _CFG_LOCAL(0x0077) //T=str, D="admin", customize admin account name
#define CSID_C_LOCAL_SSH_ENABLE		   _CFG_LOCAL(0x0078) //T=u32, D=0,  0 disable dropbear ssh , 1 enable
#define CSID_C_LOCAL_SSH_PORT		   _CFG_LOCAL(0x0079) //T=u32, D=22,  ssh service port

/* For WISP TxRx bridge function */
#define CSID_C_LOCAL_TXRXBRIDGE_ENABLE          _CFG_LOCAL(0x0080) //T=u32, D=0, TxRx Bridge enable/disable
#define CSID_C_LOCAL_TXRXBRIDGE_ACCESS_UI_IP    _CFG_LOCAL(0x0081) //T=ipv4, access ui ip
#define CSID_C_LOCAL_TXRXBRIDGE_MODE            _CFG_LOCAL(0x0082) //T=u32, D=1, TxRx Client/Server mode

#define CSID_C_LOCAL_WAN_ENABLE				_CFG_LOCAL(0x0083) //T=u32, D=1, used for adsl pvc0 in Multiwan, 0 disable, 1 enable
#define CSID_C_LOCAL_MULTIWAN_MODE			_CFG_LOCAL(0x0084) //T=u32, D=0, Multiwan Mode, 0 normal, 1 mutual
/* Reserved 16 items for Multi-WAN,  X is from 1 .... */
#define CSID_C_LOCAL_MULTIWAN_ENABLE		_CFG_LOCAL(0x0100) //T=u32, D=0, Multi WAN Enbale/Disable, Starting from MULTIWAN, Next will be WAN2 , ...
#define CSID_C_LOCAL_MULTIWAN_CLONE_MAC		_CFG_LOCAL(0x0110) //T=mac, Clone MAC of MULTIWAN port
#define CSID_C_LOCAL_MULTIWAN_WANTYPE       _CFG_LOCAL(0x0120) //T=u32, D=0, wan type (default is DHCP)
#define CSID_C_LOCAL_MULTIWAN_WEIGHT		_CFG_LOCAL(0x0130) //T=u32, define multiwan weight value

/* Half-Bridging Mode */
#define CSID_C_LOCAL_HALFBRIDGING_ENABLE    _CFG_LOCAL(0x0141) //T=u8,  Half-Bridging mode 1:enable, 0:disable

/* Reset Button */
#define CSID_C_LOCAL_RESETBUTTON_DISABLE    _CFG_LOCAL(0x0142) //T=u8,  Reset Button 1:enable, 0:disable

/* Reserved 16 items for VoIP-WAN */
#define CSID_C_LOCAL_VOIPWAN_WANTYPE       _CFG_LOCAL(0x0150) //T=u32, D=0, wan type (default is DHCP)

/* Reserved 16 items for Port Speed Setting */
#define CSID_C_LOCAL_PORT_NUM				_CFG_LOCAL(0x0160) //T=u32, Port Number (0, 1, 2, ...)
#define CSID_C_LOCAL_PORT_SPEED				_CFG_LOCAL(0x0167) //T=u32, Port Speed (10M, 100M, 1000M, ...)

/* Reserved 16 items for Ondemand  */
#define CSID_C_LOCAL_ONDEMAND_IGNO_PKT      _CFG_LOCAL(0x0170) //T=str, Record Ignore packet for ondemand (1.2.3.4/24,8080,6)

#define CSID_C_LOCAL_2ND_LANIP            _CFG_LOCAL(0x0190) //T=ipv4, D={192.168.2.1}, 2ndlan ip
#define CSID_C_LOCAL_2ND_LANNM            _CFG_LOCAL(0x0191) //T=ipv4, D={255.255.255.0}, 2ndlan subnet mask

/* IGMP snooping Index for Multiwan appliance */
#define CSID_C_LOCAL_MULTIWAN_IGMP_INTERFACE		_CFG_LOCAL(0x0199) //T=u32, define multiwan igmp index .99 localwan. 0-15 
/*  Reserved 16 items for specify each Muti-Wan interface IGMP version support */
#define CSID_C_LOCAL_MULTIWAN_IGMP_VERSION		_CFG_LOCAL(0x0200) //T=u32, define multiwan igmp version 0 auto. 1,2,3
#define CSID_C_LOCAL_WAN_IGMP_VERSION		_CFG_LOCAL(0x0210) //T=u32, define local wan IGMP version

/* Status Items */
#define CSID_S_LOCAL_LOGIN_IP           _STA_LOCAL(0x0001) //T=ipv4, ip address of logged in user
#define CSID_S_LOCAL_WAN_IP             _STA_LOCAL(0x0002) //T=ipv4, wan ip
#define CSID_S_LOCAL_WAN_NM             _STA_LOCAL(0x0003) //T=ipv4, wan netmask
#define CSID_S_LOCAL_WAN_GW             _STA_LOCAL(0x0004) //T=ipv4, wan gateway
#define CSID_S_LOCAL_WAN_PRIDNS         _STA_LOCAL(0x0005) //T=ipv4, wan primary dns
#define CSID_S_LOCAL_WAN_SECDNS         _STA_LOCAL(0x0006) //T=ipv4, wan secondary dns
#define CSID_S_LOCAL_WAN_STATUS         _STA_LOCAL(0x0007) //T=u16, wan status
#define CSID_S_LOCAL_WANTYPE_ALTERED    _STA_LOCAL(0x0008) //T=u32, ui altered
#define CSID_S_LOCAL_WANSET_ALTERED     _STA_LOCAL(0x0009) //T=u32, ui altered
#define CSID_S_LOCAL_LAN_ALTERED        _STA_LOCAL(0x000a) //T=u32, ui altered
#define CSID_S_LOCAL_WAN_IF_NAME        _STA_LOCAL(0x000b) //T=str, the interface name of wan, ppp0, eth1, ixp1
#define CSID_S_LOCAL_CNT_TIME           _STA_LOCAL(0x000c) //T=u32, the connect time
#define CSID_S_LOCAL_DETECT_WAN			_STA_LOCAL(0x000d) //T=u32, auto wantype detect result,0=no detect,1=dhcp,2=pppoe
#define CSID_S_LOCAL_WAN_CNT_FORCE		_STA_LOCAL(0x000e) //T=u32, Force WAN trigger conncet,0=disable,1=enable
#define CSID_S_LOCAL_SEP_CFG_ALTERED    _STA_LOCAL(0x000f) //T=u32, D=0, single ethernet port configuration changing, 0:no change, 1:

#define CSID_S_LOCAL_ROOT_DNS			_STA_LOCAL(0x0010) //T=ipv4, D={192.203.230.10}, ns.nasa.gov, 	Mountain View, California, U.S.
#define CSID_S_LOCAL_WAN_PORT			_STA_LOCAL(0x0020) //T=u8, WAN port cable connected, 0=disconnect, 1=connected
#define CSID_S_LOCAL_LAN_PORT			_STA_LOCAL(0x0030) //T=u8, LANx cable connected, 0=disconnect, 1=connected

/* for AGP Testing */
#define CSID_S_LOCAL_LOGINED			_STA_LOCAL(0x0040) //T=u8, logined or not?  0=not, 1=yes
#define CSID_S_LOCAL_RESETCNT			_STA_LOCAL(0x0042) //T=u16, for record reset_cnt press times
#define CSID_S_LOCAL_WPSCNT 			_STA_LOCAL(0x0044) //T=u16, for record wps_cnt press times

/* for on-demand */
#define CSID_S_LOCAL_ONDEMAND_TIMEOUT 	_STA_LOCAL(0x0045) //T=u32, record the current timeout value when ondemand

/*Device is in AP Mode, refer to CDM532RP */
#define CSID_S_LOCAL_AP_IP				_STA_LOCAL(0x0050) //T=ipv4, AP's IP
#define CSID_S_LOCAL_AP_NM				_STA_LOCAL(0x0051) //T=ipv4, AP's Netmask
#define CSID_S_LOCAL_AP_GW				_STA_LOCAL(0x0052) //T=ipv4, AP's GW
#define CSID_S_LOCAL_AP_PRIDNS			_STA_LOCAL(0x0053) //T=ipv4, AP's primary dns
#define CSID_S_LOCAL_AP_SECDNS			_STA_LOCAL(0x0054) //T=ipv4, AP's secondary dns
#define CSID_S_LOCAL_AP_STATUS         	_STA_LOCAL(0x0055) //T=u32, AP's connection status

/*Device is in STA Mode, refer to CDM532RP */
#define CSID_S_LOCAL_STA_IP				_STA_LOCAL(0x0060) //T=ipv4, STA's IP
#define CSID_S_LOCAL_STA_NM				_STA_LOCAL(0x0061) //T=ipv4, STA's Netmask
#define CSID_S_LOCAL_STA_GW				_STA_LOCAL(0x0062) //T=ipv4, STA's GW
#define CSID_S_LOCAL_STA_PRIDNS			_STA_LOCAL(0x0063) //T=ipv4, STA's primary dns
#define CSID_S_LOCAL_STA_SECDNS			_STA_LOCAL(0x0064) //T=ipv4, STA's secondary dns
#define CSID_S_LOCAL_STA_STATUS         _STA_LOCAL(0x0065) //T=u32, STA's connection status

/* For WISP TxRx bridge function */
#define CSID_S_LOCAL_TXRXBRIDGE_STATUS	_STA_LOCAL(0x0070) //T=u32, D=0, TxRx Bridge status
#define CSID_S_LOCAL_REBOOT_ALTERED     _STA_LOCAL(0x0080) //T=u32, reboot alter, 1=reboot, 0=nothing
#define CSID_S_LOCAL_FAILOVER_ALTERED   _STA_LOCAL(0x0081) //T=u32, ui altered

/* ADSL VLAN TAGE selection*/ /*refer to CDD531 */
#define CSID_S_LOCAL_ADSL_VLANNUM				_STA_LOCAL(0x0082) //T=u32, D=0, number of VLAN tage

/* Reserved 16 items for Multi-WAN,  X is from 1 .... */
#define CSID_S_LOCAL_MULTIWAN_IP				  _STA_LOCAL(0x0110) //T=ipv4, wanx ip, ex wan1 ip
#define CSID_S_LOCAL_MULTIWAN_NM            	  _STA_LOCAL(0x0120) //T=ipv4, wanx netmask
#define CSID_S_LOCAL_MULTIWAN_GW            	  _STA_LOCAL(0x0130) //T=ipv4, wanx gateway
#define CSID_S_LOCAL_MULTIWAN_PRIDNS        	  _STA_LOCAL(0x0140) //T=ipv4, wanx primary dns
#define CSID_S_LOCAL_MULTIWAN_SECDNS        	  _STA_LOCAL(0x0150) //T=ipv4, wanx secondary dns
#define CSID_S_LOCAL_MULTIWAN_STATUS        	  _STA_LOCAL(0x0160) //T=u32, wanx status
#define CSID_S_LOCAL_MULTIWAN_WANTYPE_ALTERED     _STA_LOCAL(0x0170) //T=u32, ui altered
#define CSID_S_LOCAL_MULTIWAN_WANSET_ALTERED      _STA_LOCAL(0x0180) //T=u32, ui altered
#define CSID_S_LOCAL_MULTIWAN_IF_NAME             _STA_LOCAL(0x0190) //T=str, the interface name of wan, ppp0, eth1, ixp1
#define CSID_S_LOCAL_MULTIWAN_CNT_TIME      	  _STA_LOCAL(0x01A0) //T=u32, the connect time
#define CSID_S_LOCAL_MULTIWAN_DETECT_WAN		  _STA_LOCAL(0x01B0) //T=u32, auto wantype detect result,0=no detect,1=dhcp,2=pppoe
#define CSID_S_LOCAL_MULTIWAN_CNT_FORCE			  _STA_LOCAL(0x01C0) //T=u32, Force WAN trigger conncet,0=disable,1=enable
#define CSID_S_LOCAL_MULTIWAN_CONNECTED_ALERT     _STA_LOCAL(0x01E0) //T=u32, D=0, WAN trigger conncet,1=local wan,2=multi wan
#define CSID_S_LOCAL_MULTIWAN_DISCONNECTED_ALERT  _STA_LOCAL(0x01E1) //T=u32, D=0, WAN trigger disconncet,1=local wan,2=multi wan
#define CSID_S_LOCAL_MULTIWAN_ONDEMAND_TIMEOUT    _STA_LOCAL(0x01E2) //T=u32, record the current timeout value when ondemand
#define CSID_S_LOCAL_MULTIWAN_ONDEMAND_IGNO_PKT   _STA_LOCAL(0x01E3) //T=str, Record Ignore packet for ondemand (1.2.3.4/24,8080,6)


#define CSID_S_LOCAL_HOMECOM_USER_STATUS	_STA_LOCAL(0x0200) //T=u8, D=0, Homecom user status.
#define CSID_S_LOCAL_RT8651C_SPECIAL_SNAT	_STA_LOCAL(0x0210) //T=u8, D=0, support RT8651 to connect using br0 interface

/* Reserved 16 items for VoIP-WAN,  X is from 1 .... */
#define CSID_S_LOCAL_VOIPWAN_IP	            _STA_LOCAL(0x0220) //T=ipv4, wanx ip, ex wan1 ip
#define CSID_S_LOCAL_VOIPWAN_NM            	_STA_LOCAL(0x0230) //T=ipv4, wanx netmask
#define CSID_S_LOCAL_VOIPWAN_GW            	_STA_LOCAL(0x0240) //T=ipv4, wanx gateway
#define CSID_S_LOCAL_VOIPWAN_PRIDNS        	_STA_LOCAL(0x0250) //T=ipv4, wanx primary dns
#define CSID_S_LOCAL_VOIPWAN_SECDNS        	_STA_LOCAL(0x0260) //T=ipv4, wanx secondary dns
#define CSID_S_LOCAL_VOIPWAN_STATUS        	_STA_LOCAL(0x0270) //T=u32, wanx status
#define CSID_S_LOCAL_VOIPWAN_WANTYPE_ALTERED   _STA_LOCAL(0x0280) //T=u32, ui altered
#define CSID_S_LOCAL_VOIPWAN_WANSET_ALTERED    _STA_LOCAL(0x0290) //T=u32, ui altered
#define CSID_S_LOCAL_VOIPWAN_IF_NAME       	_STA_LOCAL(0x02A0) //T=str, the interface name of wan, ppp0, eth1, ixp1
#define CSID_S_LOCAL_VOIPWAN_CNT_TIME      	_STA_LOCAL(0x02B0) //T=u32, the connect time
#define CSID_S_LOCAL_VOIPWAN_DETECT_WAN	    _STA_LOCAL(0x02C0) //T=u32, auto wantype detect result,0=no detect,1=dhcp,2=pppoe
#define CSID_S_LOCAL_VOIPWAN_CNT_FORCE	    _STA_LOCAL(0x02D0) //T=u32, Force WAN trigger conncet,0=disable,1=enable

#define CSID_S_LOCAL_WAN_IF_NAME_3G_FAILOVER	_STA_LOCAL(0x0300) //T=str, the interface name of wan used by autobak ping check, ppp0, eth1, ixp1

/* ipv6 test */
#define CSID_S_LOCAL_IPV6_PREFIX			_STA_LOCAL(0x1000)	//T=str, ipv6 prefix
#define CSID_S_LOCAL_DEF_SYSTEM_TIME        _STA_LOCAL(0x2000)  //T=str, default system time

/*  Reserved 16 items traffic tx and rx */
#define CSID_S_LOCAL_TX_TRAFFIC				_STA_LOCAL(0x3000)  //T=u32, Tx traffic
#define CSID_S_LOCAL_RX_TRAFFIC				_STA_LOCAL(0x3010)  //T=u32, Rx traffic

/*VOIP WAN IP for vapp*/
#define CSID_S_LOCAL_VOIPWAN_IP_VAPP         _STA_LOCAL(0x3020) //T=str, voip wan ip

#endif //ifndef _CSID_LOCAL_H
